@@ -1,39 +1,77 @@
+from xmlrpc.client import boolean
 from RupineHeroku.rupine_db import herokuDbAccess
 from psycopg2 import sql
 from datetime import datetime, timedelta
 import pytz
+from RupineHeroku.rupine_db.herokuDbDML import POST
 
-def postDFIDEX(connection, schema, data):
+# def postDFI(connection, schema, tableName:str, data:dict, onConflict:bool=False):
+#     columns = data.keys()
+#     onConflictString = ''
+#     if onConflict:
+#         onConflictString = 'ON CONFLICT (id) DO NOTHING'
+#     queryString = "INSERT INTO {}.{} ({}) VALUES ({}) {};".format(sql.Identifier(schema),tableName,', '.join(columns),','.join(['%s']*len(columns)),onConflictString)
 
-    query = sql.SQL("INSERT INTO {}.dfi_dex (id, key, block_number, block_timestamp, pool_reserve, reserve_a, reserve_b, dex_price) VALUES (%s,%s,%s,%s,%s,%s,%s,%s) ON CONFLICT (id) DO NOTHING;").format(sql.Identifier(schema))
-    params = (
-        data['id'],
-        data['key'],
-        data['block_number'],
-        data['block_timestamp'],
-        data['pool_reserve'],
-        data['reserve_a'],
-        data['reserve_b'],
-        data['dex_price']
-    )
-    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
-    return result
+#     params = []
+#     for key in data:
+#         params.append(data[key])
 
-def postDFIOracle(connection, schema, data):
+#     query = sql.SQL(queryString)
+#     result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
+#     return result
 
-    query = sql.SQL("INSERT INTO {}.dfi_oracle (id,key,is_live,block_number,block_median_timestamp,block_timestamp,active_price,next_price) VALUES (%s,%s,%s,%s,%s,%s,%s,%s) ON CONFLICT (id) DO NOTHING;").format(sql.Identifier(schema))
-    params = (
-        data['id'],
-        data['key'],
-        data['is_live'],
-        data['block_number'],
-        data['block_median_timestamp'],
-        data['block_timestamp'],
-        data['active_price'],
-        data['next_price']
-    )
-    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
-    return result
+# def postDFIDEX(connection, schema, data):
+#     query = sql.SQL("INSERT INTO {}.dfi_dex (id, key, key_id, block_number, block_timestamp, pool_reserve, reserve_a, reserve_b, dex_price, last_handled_block_number) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s) ON CONFLICT (id) DO NOTHING;").format(sql.Identifier(schema))
+#     params = (
+#         data['id'],
+#         data['key'],
+#         data['key_id'],
+#         data['block_number'],
+#         data['block_timestamp'],
+#         data['pool_reserve'],
+#         data['reserve_a'],
+#         data['reserve_b'],
+#         data['dex_price'],
+#         data['last_handled_block_number']
+#     )
+#     result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
+#     return result
+
+# def postDFIFee(connection, schema, data):
+
+#     query = sql.SQL("INSERT INTO {}.dfi_fee (key, key_id, block_number, block_timestamp, dex_fee_pct_token_a, dex_fee_pct_token_b, dex_fee_in_pct_token_a, dex_fee_in_pct_token_b, dex_fee_out_pct_token_a, dex_fee_out_pct_token_b, commission, last_handled_block_number) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s);").format(sql.Identifier(schema))
+#     params = (
+#         data['key'],
+#         data['key_id'],
+#         data['block_number'],
+#         data['block_timestamp'],
+#         data['dex_fee_pct_token_a'],
+#         data['dex_fee_pct_token_b'],
+#         data['dex_fee_in_pct_token_a'],
+#         data['dex_fee_in_pct_token_b'],
+#         data['dex_fee_out_pct_token_a'],
+#         data['dex_fee_out_pct_token_b'],
+#         data['commission'],
+#         data['last_handled_block_number']
+#     )
+#     result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
+#     return result
+
+# def postDFIOracle(connection, schema, data):
+
+#     query = sql.SQL("INSERT INTO {}.dfi_oracle (id,key,is_live,block_number,block_median_timestamp,block_timestamp,active_price,next_price) VALUES (%s,%s,%s,%s,%s,%s,%s,%s) ON CONFLICT (id) DO NOTHING;").format(sql.Identifier(schema))
+#     params = (
+#         data['id'],
+#         data['key'],
+#         data['is_live'],
+#         data['block_number'],
+#         data['block_median_timestamp'],
+#         data['block_timestamp'],
+#         data['active_price'],
+#         data['next_price']
+#     )
+#     result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
+#     return result
 
 # TODO
 # getDEXPrice for timestamp: exact (nearest timestamp) OR hourly, daily, monthly-  -> group and return min, max, open, close, avg, median
@@ -70,10 +108,21 @@ def getlatestOracleBlock(connection, schema, tokenSymbol:str):
 
 def getlatestDEXBlock(connection, schema, poolSymbol:str=None):
     if poolSymbol is None:
-        query = sql.SQL("SELECT MAX(d.block_number) AS max_block_number FROM {0}.dfi_dex d").format(sql.Identifier(schema))
+        query = sql.SQL("SELECT d.key,d.key_id,MAX(d.last_handled_block_number) AS max_block_number, FIRST_VALUE(d.id) OVER (PARTITION d.key,d.key_id BY ORDER BY d.last_handled_block_number DESC) AS max_id FROM {0}.dfi_dex d GROUP BY d.key,d.key_id").format(sql.Identifier(schema))
         params = []
     else:
-        query = sql.SQL("SELECT MAX(d.block_number) AS max_block_number FROM {0}.dfi_dex d WHERE 1=1 AND key = %s ").format(sql.Identifier(schema))
+        query = sql.SQL("SELECT d.key,d.key_id,MAX(d.last_handled_block_number) AS max_block_number, FIRST_VALUE(d.id) OVER (PARTITION d.key,d.key_id BY ORDER BY d.last_handled_block_number DESC) AS max_id FROM {0}.dfi_dex d WHERE 1=1 AND key = %s GROUP BY d.key,d.key_id").format(sql.Identifier(schema))
+        params = [poolSymbol]
+   
+    result = herokuDbAccess.fetchDataInDatabase(query, params, connection)    
+    return result
+
+def getlatestDEXFeeBlock(connection, schema, poolSymbol:str=None):
+    if poolSymbol is None:
+        query = sql.SQL("SELECT d.key,d.key_id,MAX(d.last_handled_block_number) AS max_block_number, FIRST_VALUE(d.id) OVER (PARTITION d.key,d.key_id BY ORDER BY d.last_handled_block_number DESC) AS max_id FROM {0}.dfi_fee d GROUP BY d.key,d.key_id").format(sql.Identifier(schema))
+        params = []
+    else:
+        query = sql.SQL("SELECT d.key,d.key_id,MAX(d.last_handled_block_number) AS max_block_number, FIRST_VALUE(d.id) OVER (PARTITION d.key,d.key_id BY ORDER BY d.last_handled_block_number DESC) AS max_id FROM {0}.dfi_fee d WHERE 1=1 AND key = %s GROUP BY d.key,d.key_id").format(sql.Identifier(schema))
         params = [poolSymbol]
    
     result = herokuDbAccess.fetchDataInDatabase(query, params, connection)    
@@ -106,24 +155,28 @@ def postDFIBotEvent(connection, schema, data):
     if 'risk' not in data:
         data['risk'] = None
     
-    query = sql.SQL("INSERT INTO {}.dfi_bot (id,address,block_number,future_settlement_block,loan,loan_token,loan_oracle_price,loan_dex_price,invest_type,sentiment,risk,is_active,waiting_for_loan_payback) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s);").format(sql.Identifier(schema))
-    params = (
-        data['id'],
-        data['address'],
-        data['block_number'],
-        data['future_settlement_block'],
-        data['loan'],
-        data['loan_token'],
-        data['loan_oracle_price'],
-        data['loan_dex_price'],
-        data['invest_type'],
-        data['sentiment'],
-        data['risk'],
-        'Y',
-        'N'
-    )
-    result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
-    return result
+    data['is_active'] = 'Y'
+    data['waiting_for_loan_payback'] = 'N'
+    
+    return POST(connection,schema,'dfi_bot',data)
+    # query = sql.SQL("INSERT INTO {}.dfi_bot (id,address,block_number,future_settlement_block,loan,loan_token,loan_oracle_price,loan_dex_price,invest_type,sentiment,risk,is_active,waiting_for_loan_payback) VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s);").format(sql.Identifier(schema))
+    # params = (
+    #     data['id'],
+    #     data['address'],
+    #     data['block_number'],
+    #     data['future_settlement_block'],
+    #     data['loan'],
+    #     data['loan_token'],
+    #     data['loan_oracle_price'],
+    #     data['loan_dex_price'],
+    #     data['invest_type'],
+    #     data['sentiment'],
+    #     data['risk'],
+    #     'Y',
+    #     'N'
+    # )
+    # result = herokuDbAccess.insertDataIntoDatabase(query, params, connection)    
+    # return result
 
 def putDFIBotEventInvest(connection, schema, data):
     query = sql.SQL("UPDATE {}.dfi_bot SET invest = %s, invest_token = %s, invest_oracle_price =%s, invest_dex_price = %s WHERE id = %s AND address = %s").format(sql.Identifier(schema))
@@ -276,6 +329,7 @@ def getLatestBlockOfBotEvent(connection, schema, address:str):
 #         os.environ.get("HEROKU_DB_PORT"),
 #         os.environ.get("HEROKU_DB_DATABASE")
 #     )
+#     print(POST(connection,'dbdev','my_table',{'id':1,'col':'pubs'},True))
 #     data = {
 #         'id':"bla",
 #         'key':"TSLA-DUSD",
