@@ -37,46 +37,34 @@ def getAccountAddress(connection, schema, account=None, public_address=None, cha
     result = herokuDbAccess.fetchDataInDatabase(query, [*params], connection)  
     return result
 
+def getUniqueTokenFromTransactions(connection, schema, public_address:str=None, chain_id:int=None):
+    
+    queryStrSend = "select distinct(amount_send_dwh_token_id) \
+        FROM {}.dwh_transaction \
+        WHERE amount_send_dwh_token_id is not NULL"
 
-# import os
-# from dotenv import load_dotenv
-# import herokuDbAccess as db
-# from datetime import datetime
-# load_dotenv()
+    queryStrRecv = "select distinct(amount_recv_dwh_token_id) \
+        FROM {}.dwh_transaction \
+        WHERE amount_recv_dwh_token_id is not NULL"
 
-# if __name__ == '__main__':
-#     connection = db.connect(
-#         os.environ.get("HEROKU_DB_USER"),
-#         os.environ.get("HEROKU_DB_PW"),
-#         os.environ.get("HEROKU_DB_HOST"),
-#         os.environ.get("HEROKU_DB_PORT"),
-#         os.environ.get("HEROKU_DB_DATABASE")
-#     )
-#     data = {
-#         'chain_id':None,
-#         'chain':'DeFiChain',
-#         'account':'Benni',
-#         'public_address':'df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct',
-#         'context':'private'
-        
-#     }
-#     postAccountAddress(connection,'stage',data)
-#     print(getAccountAddress(connection,'stage','Benni',chain='DeFiChain'))
-    # data = {
-    #     'chain_id':1,
-    #     'chain':'ETH',
-    #     'account':'MYACC',
-    #     'address':'dfi1',
-    #     'token':'BLUE1',
-    #     'buy_timestamp':127897934,
-    #     'buy_price':12.4,
-    #     'buy_transaction_hashes':'sjdlfjskljdfklsj,jhghjhghj',
-    #     'sell_timestamp':893434,
-    #     'sell_price':34.35,
-    #     'sell_transaction_hashes':'shdsdfjkhsdjkf2,sdjfklj',
-    #     'amount':12.2
-    # }
-    #res = getTaxTransactionRewards(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None,'DFI',0)
-    #res = getTaxTokens(connection,'stage',['dfi1','df1qgssyvcqld7cwxzvnajyt3xlqae67muxrqh06ct'],'DeFiChain',None)
-    # res = getLatestWarehouseDate(connection,'stage','BLUE12','MYACC','Defi',None)
-    # print(res)
\ No newline at end of file
+    params = []
+
+    if chain_id:
+        queryStrSend = queryStrSend + " AND chain_id=%s"
+        queryStrRecv = queryStrRecv + " AND chain_id=%s"
+        params.append(chain_id)
+
+    if public_address:
+        queryStrSend = queryStrSend + " AND public_address=%s"
+        queryStrRecv = queryStrRecv + " AND public_address=%s"
+        params.append(public_address)
+
+    querySend = sql.SQL(queryStrSend).format(sql.Identifier(schema))
+    queryRecv = sql.SQL(queryStrRecv).format(sql.Identifier(schema))
+
+    resultSend = herokuDbAccess.fetchDataInDatabase(querySend, params, connection)    
+    resultRecv = herokuDbAccess.fetchDataInDatabase(queryRecv, params, connection)    
+
+    allResults = [*resultSend, *resultRecv]
+
+    return allResults
\ No newline at end of file
