@@ -140,10 +140,10 @@ def PUT_BULK(connection, schema, updates:list, tableName:str, conditionColumns:l
         conditionArray.append("t.{} = tmp.{}".format(col,col))
     for col in columns:
         if col not in conditionColumns:
-            setArray.append("SET t.{} = tmp.{}".format(col,col))
+            setArray.append("{} = tmp.{}".format(col,col))
     
     tmpTableName = schema + '_tmp_' + tableName
-    queryString = "UPDATE {{}}.{} t {} FROM {} tmp WHERE 1=1 AND {}".format(tableName,', '.join(setArray),tmpTableName,' AND '.join(conditionArray))
+    queryString = "UPDATE {{}}.{} AS t SET {} FROM {} tmp WHERE 1=1 AND {}".format(tableName,', '.join(setArray),tmpTableName,' AND '.join(conditionArray))
 
     query = sql.SQL(queryString).format(sql.Identifier(schema))
 
