@@ -2,9 +2,9 @@ from psycopg2 import sql
 from ..DataStructures import ObjEvmToken
 from . import herokuDbAccess
 
-def postEvmToken(connection, schema:str, chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name):
+def postEvmLiquidityPool(connection, schema:str, chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name):
 
-    query = sql.SQL("INSERT INTO {}.evm_token (chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name) \
+    query = sql.SQL("INSERT INTO {}.evm_token_liquidity_pools (chain_id, liquidity_pool_address, token_address_a, token_address_b, exchange_name) \
            VALUES (%s,%s,%s,%s,%s)").format(sql.Identifier(schema))
     params = (
         chain_id,
